
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.const.all;

entity icache is
port(clk, reset: in std_logic;
	addr: in adr_type;
	ins: out dat_type;
	tlb_addr: in adr_type;

	miss: out std_logic;
	mem_line: in line_dat_type;
	memfill: in std_logic;
	mem_addr: out adr_type;
	excep: out std_logic;
	tlb_miss: in std_logic
);
end entity icache;

architecture rtl of icache is
	type cache_dat_type is array (0 to 3) of line_dat_type;
	type cache_tag_type is array (0 to 3) of line_tag_type;
	type cache_valid_type is array (0 to 3) of std_logic;

	signal ary_valid: cache_valid_type;
	signal ary_tag: cache_tag_type;
	signal ary_data: cache_dat_type;

	signal i_miss, valid, fill: std_logic;
	signal line: line_dat_type;
	signal itag, tag: line_tag_type;
	signal ncol, nrow: Natural;
	signal sel: std_logic_vector(3 downto 0);
begin
	excep <= addr(0) or addr(1);
	nrow <= conv_integer(addr(line_row_b downto line_row_e));
	itag <= tlb_addr(line_tag_b downto line_tag_e);
	ncol <= conv_integer(addr(line_col_b downto line_col_e));

	valid <= ary_valid(nrow);
	tag <= ary_tag(nrow);
	line <= ary_data(nrow);
	ins <= line(ncol);

	i_miss <= '1' when valid = '0' or tag /= itag else
	          '0';
	miss <= i_miss and not tlb_miss;
	fill <= i_miss and memfill;
	mem_addr(adr_w-1 downto line_col_w) <= addr(adr_w-1 downto line_col_w);
	mem_addr(line_col_w-1 downto 0) <= (others => '0');

	defcach: for i in 0 to 3 generate
		sel(i) <= '1' when nrow = i and fill = '1' else '0';

		defline: process(clk, reset, mem_line, fill, sel(i))
		begin
			if reset = '1' then
				ary_valid(i) <= '0';
				ary_tag(i) <= (others => '0');
				ary_data(i) <= (others => (others => '0'));
			elsif clk = '1' and clk'event and sel(i) = '1' then
				ary_valid(i) <= '1';
				ary_tag(i) <= itag;
				ary_data(i) <= mem_line;
			end if;
		end process;
	end generate;

end architecture rtl;

