library ieee;
use ieee.std_logic_1164.all;

use work.const.all;


entity id_latch is
port (
	clk: in std_logic;
	reset: in std_logic;
	stall: in std_logic;
	nop: in std_logic;
	--in
	alu_op_in: in std_logic_vector(3 downto 0);
	sel8_in, sel16_in, sel32_in : in std_logic;
	isValid_in, isMem_in, isLdi_in, isAlu_in, isAluSpec_in: in std_logic;
	isHigh_in, isLow_in, isStore_in, isBranch_in, isHalt: in std_logic;
	rdAdr_in, rs1Adr_in, rs2Adr_in: in reg_adr_type;
	imm_in : in dat_type;
	branch_cond_in: in std_logic_vector(1 downto 0);
	pc_in, branch_adr_in: in adr_type;
	rs1_in, rs2_in: in dat_type;
	exc_vector_in: in exc_vector_type;
	itlb_ena_in: in std_logic;
	--out
	alu_op_out: out std_logic_vector(3 downto 0);
	sel8_out, sel16_out, sel32_out : out std_logic;
	isValid_out, isMem_out, isLdi_out, isAlu_out, isAluSpec_out: out std_logic;
	isHigh_out, isLow_out, isStore_out, isBranch_out, isHalt_out: out std_logic;
	rdAdr_out, rs1Adr_out, rs2Adr_out: out reg_adr_type;
	imm_out : out dat_type;
	branch_cond_out: out std_logic_vector(1 downto 0);
	pc_out, branch_adr_out: out adr_type;
	rs1_out, rs2_out: out dat_type;
	exc_vector_out: out exc_vector_type;
	itlb_ena_out: out std_logic
);
end entity id_latch;

architecture behav of id_latch is
begin
	latch:process(clk, reset, stall, nop) is
	begin
		if reset = '1' then
			isValid_out <= '0';
			isMem_out  <= '0';
			isLdi_out  <= '0';
			isAlu_out  <= '0';
			isAluSpec_out <= '0';
			isHigh_out <= '0';
			isLow_out <= '0';
			isStore_out <= '0';
			isBranch_out <= '0';
			isHalt_out <= '0';
			sel8_out   <= '0';
			sel16_out  <= '0';
			sel32_out  <= '0';
			branch_cond_out <= (others => '0');
			branch_adr_out <= (others => '0');
			imm_out <= (others => '0');
			alu_op_out <= (others => '0');
			rs1_out <= (others => '0');
			rs2_out <= (others => '0');
			rdAdr_out  <= (others => '0');
			rs1Adr_out <= (others => '0');
			rs2Adr_out <= (others => '0');
			pc_out <= (others => '0');
			exc_vector_out <= (others => '0');
			itlb_ena_out <= '0';
		elsif clk = '1' and clk'event and stall = '0' then
			if nop = '1' then
				isValid_out <= '0';
				isMem_out  <= '0';
				isLdi_out  <= '0';
				isAlu_out  <= '0';
				isAluSpec_out <= '0';
				isHigh_out <= '0';
				isLow_out <= '0';
				isStore_out <= '0';
				isBranch_out <= '0';
				isHalt_out <= '0';
				exc_vector_out <= (others => '0');
				itlb_ena_out <= '0';
			else
				isValid_out <= isValid_in;
				isMem_out  <= isMem_in;
				isLdi_out  <= isLdi_in;
				isAlu_out  <= isAlu_in;
				isAluSpec_out <= isAluSpec_in;
				isHigh_out <= isHigh_in;
				isLow_out <= isLow_in;
				isStore_out <= isStore_in;
				isBranch_out <= isBranch_in;
				isHalt_out <= isHalt;
				exc_vector_out <= exc_vector_in;
				itlb_ena_out <= itlb_ena_in;
			end if;
			pc_out <= pc_in;
			sel8_out   <= sel8_in;
			sel16_out  <= sel16_in;
			sel32_out  <= sel32_in;
			branch_cond_out <= branch_cond_in;
			branch_adr_out <= branch_adr_in;
			imm_out <= imm_in;
			alu_op_out <= alu_op_in;
			rs1_out <= rs1_in;
			rs2_out <= rs2_in;
			rdAdr_out  <= rdAdr_in;
			rs1Adr_out <= rs1Adr_in;
			rs2Adr_out <= rs2Adr_in;
		end if;
	end process latch;
end architecture behav;
