library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.const.all;



entity id_reg_file is
port (
	clk: std_logic;
	reset: std_logic;

	wr_ena: in std_logic;
	rs1Adr: in reg_adr_type;
	rs2Adr: in reg_adr_type;
	wbrdAdr: in reg_adr_type;
	wbrdDat: in dat_type;

	rs1Dat: out dat_type;
	rs2Dat: out dat_type
);
end entity id_reg_file;

architecture behav of id_reg_file is
	type reg_file_type is array (0 to 15) of dat_type;
	signal regs: reg_file_type;
	signal sel: std_logic_vector(15 downto 1);
	signal i, wr_adr: integer;
begin

	wr_adr <= conv_integer(wbrdAdr);

	regs(0) <= (others => '0');
	bank: for i in 1 to 15 generate
		sel(i) <= '1' when wr_adr = i and wr_ena = '1' else
		          '0';

		reg: process(clk, reset, sel(i)) is
		begin
			if reset = '1' then
				regs(i) <= (others=> '0');
			elsif clk = '0' and clk'event and sel(i) = '1' then
				regs(i) <= wbrdDat;
			end if;
		end process reg;
	end generate;

	rs1Dat <= regs(conv_integer(rs1Adr));
	rs2Dat <= regs(conv_integer(rs2Adr));

end architecture behav;
