library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.const.all;



entity if_next_pc is
port (
	clk: in std_logic;
	reset: in std_logic;
	wr_enable: in std_logic;

	id_isJump: in std_logic; --take jump address?
	id_jumpAdr: in adr_type;
	ex_takeBranch: in std_logic; --take branch?
	ex_isBranch: in std_logic;
	ex_brAdr: in adr_type;
	id_trapPC: in adr_type;
	wb_trapPC_offset: in adr_type;
	wb_isExc: in std_logic;

	prev_pc_out: out adr_type;
	curr_pc_out: out adr_type
);
end entity if_next_pc;

architecture behav of if_next_pc is
	signal pc, prev_pc: adr_type;
	signal excPC: adr_type; -- trapPc + offset
	signal powerup, ena_pc: std_logic;
begin
	excPC <= id_trapPC + wb_trapPC_offset;

	pc <= excPC when wb_isExc = '1' else
	     id_jumpAdr when id_isJump = '1' else
	     ex_brAdr when ex_takeBranch = '1' and ex_isBranch = '1' else
	     prev_pc + 4 when powerup = '1' else
	     (others => '0');

	ena_pc <= wr_enable and powerup;

	defpow: process(reset, clk, wr_enable)
	begin
		if reset = '1' then
			powerup <= '0';
		elsif clk = '1' and clk'event and wr_enable = '1' then
			powerup <= '1';
		end if;
	end process;


	defpc: process(reset, clk, ena_pc, pc)
	begin
		if reset = '1' then
			prev_pc <= (others => '0');
		elsif clk = '1' and clk'event and ena_pc = '1' then
			prev_pc <= pc;
		end if;
	end process;

	curr_pc_out <= pc;
	prev_pc_out <= prev_pc;
end architecture behav;
