library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use work.const.all;



entity if_stage is
port (
	clk: in std_logic;
	reset: in std_logic;
	stall, nop: in std_logic;

	id_isJump: in std_logic;
	id_jmpAdr: in adr_type;
	ex_takeBranch: in std_logic;
	ex_isBranch: in std_logic;
	ex_brAdr: in adr_type;
	id_trapPC: in adr_type;
	wb_trapPc_Offset: in adr_type;
	wb_isExc: in std_logic;
	tlb_data: in dat_type; --from ID
	tlb_adr: in adr_type;
	tlb_wrEna: in std_logic;

	if_PC_out: out adr_type;
	ir_out: out dat_type;

	exc_vector_out: out exc_vector_type;

	o_isValid: out std_logic;

	mem_addr: out adr_type;
	mem_data: in line_dat_type;
	miss: out std_logic;
	fill: in std_logic
);
end entity if_stage;

architecture behav of if_stage is
	signal curr_pc, prev_pc, pc_phys: adr_type;
	signal if_ir: dat_type;
	signal pc_wr_enable: std_logic;
	signal exc_vector, exc_vector_align, exc_vector_access: exc_vector_type;
	signal exc_pc_align, exc_itlb_miss, exc_itlb_access, itlb_access, itlb_miss: std_logic;
	signal exc_align, cache_miss: std_logic;
	--signal tlb_data: dat_type;
	--signal tlb_adr: adr_type;
	--signal tlb_wrEna: std_logic;
begin

	pc_wr_enable <= not stall;

	if_pc: entity work.if_next_pc(behav)
		PORT MAP(
					clk,
					reset,
					pc_wr_enable,

					id_isJump, id_jmpAdr,
					ex_takeBranch, ex_isBranch, ex_brAdr,
					id_trapPC, wb_trapPc_Offset, wb_isExc,

					prev_pc, curr_pc
		);
	if_PC_out <= prev_pc; --PC register is the 'latch' at the same time


	exc_pc_align <= '1' when curr_pc(0) = '1' or curr_pc(1) = '1' else
					'0';
	exc_inj_pc_align: entity work.exc_inject(behav)
		GENERIC MAP(exc_if_align)
		PORT MAP(
			exc_pc_align,
			(others => '0'),
			exc_vector_align
		);

	exc_itlb_access <= itlb_access and not exc_pc_align;
	exc_inj_access: entity work.exc_inject(behav)
		GENERIC MAP(exc_if_access)
		PORT MAP(
			exc_itlb_access,
			exc_vector_align,
			exc_vector_access
		);

	exc_itlb_miss <= itlb_miss and not exc_itlb_access;
	exc_inj_miss: entity work.exc_inject(behav)
		GENERIC MAP(exc_if_itlb)
		PORT MAP(
			exc_itlb_miss,
			exc_vector_access,
			exc_vector
		);

	itlb: entity work.tlb(behav)
		PORT MAP(
			clk, reset,
			curr_pc,     --change to curr_pc probably to fix; TRUE
			'0','0','1', --rwx - in IF always only execute
			tlb_data,
			tlb_adr,
			tlb_wrEna,
			pc_phys,
			itlb_miss, itlb_access
		);

	miss <= cache_miss;
	cache: entity work.icache(rtl)
		PORT MAP(clk, reset,
		         curr_pc, if_ir, pc_phys,
		         cache_miss, mem_data, fill, mem_addr,
		         exc_align, exc_itlb_miss);

	latch: entity work.if_latch(behav)
		PORT MAP(
			clk,
			reset,
			stall,
			nop,
			if_ir,
			exc_vector,
			'1', --isValid
			ir_out,
			exc_vector_out,
			o_isValid
		);
end architecture behav;
