library ieee;
use ieee.std_logic_1164.all;

use work.const.all;


entity mem_latch is
port (
	clk: in std_logic;
	reset: in std_logic;
	stall: in std_logic;
	nop: in std_logic;
	i_mem_data: in dat_type;
	o_mem_data: out dat_type;

	i_pc: in adr_type;
	i_branch_cond: in std_logic_vector(1 downto 0);
	i_isValid, i_ismem, i_ldi: in std_logic;
	i_isalu, i_isaluspec, i_isstore, i_ishalt: in std_logic;
	i_reg_addr: in reg_adr_type;
	i_alu_data: in dat_type;
	i_regWrEna: in std_logic;
	i_exc_vector: in exc_vector_type;
	i_rmAddress: in adr_type;

	o_pc: out adr_type;
	o_branch_cond: out std_logic_vector(1 downto 0);
	o_isValid, o_ismem, o_ldi: out std_logic;
	o_isalu, o_isaluspec, o_isstore, o_ishalt: out std_logic;
	o_reg_addr: out  reg_adr_type;
	o_alu_data: out dat_type;
	o_regWrEna: out std_logic;
	o_exc_vector: out exc_vector_type;
	o_rmAddress: out adr_type
);
end entity mem_latch;

architecture behav of mem_latch is
begin
	MEM_LATCH: process(clk, reset, stall, nop)
	begin
		if reset = '1' then
			o_branch_cond <= (others => '0');
			o_isValid <= '0';
			o_ismem <= '0';
			o_ldi <= '0';
			o_isalu <= '0';
			o_isaluspec <= '0';
			o_isstore <= '0';
			o_ishalt <= '0';
			o_mem_data <= (others => '0');
			o_reg_addr <= (others => '0');
			o_alu_data <= (others => '0');
			o_regWrEna <= '0';
			o_pc <= (others => '0');
			o_exc_vector <= (others => '0');
			o_rmAddress <= (others => '0');
		elsif clk = '1' and clk'event and stall = '0' then
			if nop = '1' then
				o_isValid <= '0';
				o_ismem <= '0';
				o_ldi <= '0';
				o_isalu <= '0';
				o_isaluspec <= '0';
				o_isstore <= '0';
				o_ishalt <= '0';
				o_exc_vector <= (others => '0');
			else
				o_isValid <= i_isValid;
				o_ismem <= i_ismem;
				o_ldi <= i_ldi;
				o_isalu <= i_isalu;
				o_isaluspec <= i_isaluspec;
				o_isstore <= i_isstore;
				o_ishalt <= i_ishalt;
				o_exc_vector <= i_exc_vector;
			end if;
			o_pc <= i_pc;
			o_branch_cond <= i_branch_cond;
			o_alu_data <= i_alu_data;
			o_mem_data <= i_mem_data;
			o_reg_addr <= i_reg_addr;
			o_regWrEna <= i_regWrEna;
			o_rmAddress <= i_rmAddress;
		end if;
	end process;

end architecture behav;

