
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;


use work.const.all;

entity memdec is
	port (
		cs, wr, rd: in std_logic;
		addr: in adr_type;
		odata: out dat_type;

		wr_ram: out std_logic;
		ramadr: out ram_adr_type;
		ramdata: in dat_type;

		wr_rom: out std_logic;
		romadr: out rom_adr_type;
		romdata: in dat_type
);
end entity memdec;

architecture rtl of memdec is
	signal high_adr: integer;
	signal ramsel, romsel, good: std_logic;
begin
	good <= '1' when conv_integer(addr(adr_w-1 downto mem_w)) = 0 else
	        '0';
	ramsel <= cs and addr(mem_w-1) and good;
	romsel <= cs and not addr(mem_w-1) and good;
	ramadr <= addr(ram_w-1 downto 0);
	romadr <= addr(rom_w-1 downto 0);

	odata <= ramdata when ramsel = '1' and rd = '1' else
	         romdata when romsel = '1' and rd = '1' else
	         (others => 'X');

	wr_ram <= wr and ramsel;
	wr_rom <= '0';
end architecture rtl;
